@@ -19,17 +19,27 @@
 #include <ctype.h>
 #include <float.h>
 #include <limits.h>
+#include <linux/ptp_clock.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include "as_capable.h"
+#include "bmc.h"
+#include "clock.h"
 #include "config.h"
 #include "ether.h"
 #include "hash.h"
 #include "print.h"
 #include "util.h"
 
+struct interface {
+	STAILQ_ENTRY(interface) list;
+};
+
 enum config_section {
 	GLOBAL_SECTION,
+	UC_MTAB_SECTION,
 	PORT_SECTION,
 	UNKNOWN_SECTION,
 };
@@ -133,6 +143,20 @@ static struct config_enum clock_servo_enu[] = {
 	{ NULL, 0 },
 };
 
+static struct config_enum clock_type_enu[] = {
+	{ "OC",      CLOCK_TYPE_ORDINARY },
+	{ "BC",      CLOCK_TYPE_BOUNDARY },
+	{ "P2P_TC",  CLOCK_TYPE_P2P      },
+	{ "E2E_TC",  CLOCK_TYPE_E2E      },
+	{ NULL, 0 },
+};
+
+static struct config_enum dataset_comp_enu[] = {
+	{ "ieee1588", DS_CMP_IEEE1588 },
+	{ "G.8275.x", DS_CMP_G8275    },
+	{ NULL, 0 },
+};
+
 static struct config_enum delay_filter_enu[] = {
 	{ "moving_average", FILTER_MOVING_AVERAGE },
 	{ "moving_median",  FILTER_MOVING_MEDIAN  },
@@ -146,6 +170,20 @@ static struct config_enum delay_mech_enu[] = {
 	{ NULL, 0 },
 };
 
+static struct config_enum extts_polarity_enu[] = {
+	{ "rising",  PTP_RISING_EDGE  },
+	{ "falling", PTP_FALLING_EDGE },
+	{ "both",    PTP_RISING_EDGE | PTP_FALLING_EDGE },
+	{ NULL, 0 },
+};
+
+static struct config_enum hwts_filter_enu[] = {
+	{ "normal",  HWTS_FILTER_NORMAL  },
+	{ "check",   HWTS_FILTER_CHECK   },
+	{ "full",    HWTS_FILTER_FULL    },
+	{ NULL, 0 },
+};
+
 static struct config_enum nw_trans_enu[] = {
 	{ "L2",    TRANS_IEEE_802_3 },
 	{ "UDPv4", TRANS_UDP_IPV4   },
@@ -170,14 +208,31 @@ static struct config_enum tsproc_enu[] = {
 	{ NULL, 0 },
 };
 
+static struct config_enum as_capable_enu[] = {
+	{ "true", AS_CAPABLE_TRUE },
+	{ "auto", AS_CAPABLE_AUTO },
+	{ NULL, 0 },
+};
+
+static struct config_enum bmca_enu[] = {
+	{ "ptp",  BMCA_PTP  },
+	{ "noop", BMCA_NOOP },
+	{ NULL, 0 },
+};
+
 struct config_item config_tab[] = {
 	PORT_ITEM_INT("announceReceiptTimeout", 3, 2, UINT8_MAX),
+	PORT_ITEM_ENU("asCapable", AS_CAPABLE_AUTO, as_capable_enu),
 	GLOB_ITEM_INT("assume_two_step", 0, 0, 1),
 	PORT_ITEM_INT("boundary_clock_jbod", 0, 0, 1),
+	PORT_ITEM_ENU("BMCA", BMCA_PTP, bmca_enu),
 	GLOB_ITEM_INT("check_fup_sync", 0, 0, 1),
 	GLOB_ITEM_INT("clockAccuracy", 0xfe, 0, UINT8_MAX),
 	GLOB_ITEM_INT("clockClass", 248, 0, UINT8_MAX),
+	GLOB_ITEM_STR("clockIdentity", "000000.0000.000000"),
 	GLOB_ITEM_ENU("clock_servo", CLOCK_SERVO_PI, clock_servo_enu),
+	GLOB_ITEM_ENU("clock_type", CLOCK_TYPE_ORDINARY, clock_type_enu),
+	GLOB_ITEM_ENU("dataset_comparison", DS_CMP_IEEE1588, dataset_comp_enu),
 	PORT_ITEM_INT("delayAsymmetry", 0, INT_MIN, INT_MAX),
 	PORT_ITEM_ENU("delay_filter", FILTER_MOVING_MEDIAN, delay_filter_enu),
 	PORT_ITEM_INT("delay_filter_length", 10, 1, INT_MAX),
@@ -192,26 +247,39 @@ struct config_item config_tab[] = {
 	PORT_ITEM_INT("follow_up_info", 0, 0, 1),
 	GLOB_ITEM_INT("free_running", 0, 0, 1),
 	PORT_ITEM_INT("freq_est_interval", 1, 0, INT_MAX),
+	GLOB_ITEM_INT("G.8275.defaultDS.localPriority", 128, 1, UINT8_MAX),
+	PORT_ITEM_INT("G.8275.portDS.localPriority", 128, 1, UINT8_MAX),
 	GLOB_ITEM_INT("gmCapable", 1, 0, 1),
+	GLOB_ITEM_ENU("hwts_filter", HWTS_FILTER_NORMAL, hwts_filter_enu),
 	PORT_ITEM_INT("hybrid_e2e", 0, 0, 1),
+	PORT_ITEM_INT("ignore_source_id", 0, 0, 1),
 	PORT_ITEM_INT("ignore_transport_specific", 0, 0, 1),
 	PORT_ITEM_INT("ingressLatency", 0, INT_MIN, INT_MAX),
+	PORT_ITEM_INT("inhibit_announce", 0, 0, 1),
+	PORT_ITEM_INT("inhibit_delay_req", 0, 0, 1),
+	PORT_ITEM_INT("inhibit_multicast_service", 0, 0, 1),
 	GLOB_ITEM_INT("initial_delay", 0, 0, INT_MAX),
 	GLOB_ITEM_INT("kernel_leap", 1, 0, 1),
+	GLOB_ITEM_STR("leapfile", NULL),
 	PORT_ITEM_INT("logAnnounceInterval", 1, INT8_MIN, INT8_MAX),
 	PORT_ITEM_INT("logMinDelayReqInterval", 0, INT8_MIN, INT8_MAX),
 	PORT_ITEM_INT("logMinPdelayReqInterval", 0, INT8_MIN, INT8_MAX),
 	PORT_ITEM_INT("logSyncInterval", 0, INT8_MIN, INT8_MAX),
 	GLOB_ITEM_INT("logging_level", LOG_INFO, PRINT_LEVEL_MIN, PRINT_LEVEL_MAX),
+	PORT_ITEM_INT("masterOnly", 0, 0, 1),
+	GLOB_ITEM_INT("maxStepsRemoved", 255, 2, UINT8_MAX),
 	GLOB_ITEM_STR("message_tag", NULL),
 	GLOB_ITEM_STR("manufacturerIdentity", "00:00:00"),
 	GLOB_ITEM_INT("max_frequency", 900000000, 0, INT_MAX),
 	PORT_ITEM_INT("min_neighbor_prop_delay", -20000000, INT_MIN, -1),
+	PORT_ITEM_INT("msg_interval_request", 0, 0, 1),
 	PORT_ITEM_INT("neighborPropDelayThresh", 20000000, 0, INT_MAX),
 	PORT_ITEM_INT("net_sync_monitor", 0, 0, 1),
 	PORT_ITEM_ENU("network_transport", TRANS_UDP_IPV4, nw_trans_enu),
 	GLOB_ITEM_INT("ntpshm_segment", 0, INT_MIN, INT_MAX),
 	GLOB_ITEM_INT("offsetScaledLogVariance", 0xffff, 0, UINT16_MAX),
+	PORT_ITEM_INT("operLogPdelayReqInterval", 0, INT8_MIN, INT8_MAX),
+	PORT_ITEM_INT("operLogSyncInterval", 0, INT8_MIN, INT8_MAX),
 	PORT_ITEM_INT("path_trace_enabled", 0, 0, 1),
 	GLOB_ITEM_DBL("pi_integral_const", 0.0, 0.0, DBL_MAX),
 	GLOB_ITEM_DBL("pi_integral_exponent", 0.4, -DBL_MAX, DBL_MAX),
@@ -228,25 +296,45 @@ struct config_item config_tab[] = {
 	PORT_ITEM_STR("p2p_dst_mac", "01:80:C2:00:00:0E"),
 	GLOB_ITEM_STR("revisionData", ";;"),
 	GLOB_ITEM_INT("sanity_freq_limit", 200000000, 0, INT_MAX),
+	GLOB_ITEM_INT("servo_num_offset_values", 10, 0, INT_MAX),
+	GLOB_ITEM_INT("servo_offset_threshold", 0, 0, INT_MAX),
+	GLOB_ITEM_STR("slave_event_monitor", ""),
 	GLOB_ITEM_INT("slaveOnly", 0, 0, 1),
+	GLOB_ITEM_INT("socket_priority", 0, 0, 15),
 	GLOB_ITEM_DBL("step_threshold", 0.0, 0.0, DBL_MAX),
 	GLOB_ITEM_INT("summary_interval", 0, INT_MIN, INT_MAX),
 	PORT_ITEM_INT("syncReceiptTimeout", 0, 0, UINT8_MAX),
+	GLOB_ITEM_INT("tc_spanning_tree", 0, 0, 1),
 	GLOB_ITEM_INT("timeSource", INTERNAL_OSCILLATOR, 0x10, 0xfe),
 	GLOB_ITEM_ENU("time_stamping", TS_HARDWARE, timestamping_enu),
 	PORT_ITEM_INT("transportSpecific", 0, 0, 0x0F),
+	PORT_ITEM_INT("ts2phc.channel", 0, 0, INT_MAX),
+	PORT_ITEM_INT("ts2phc.extts_correction", 0, INT_MIN, INT_MAX),
+	PORT_ITEM_ENU("ts2phc.extts_polarity", PTP_RISING_EDGE, extts_polarity_enu),
+	PORT_ITEM_INT("ts2phc.master", 0, 0, 1),
+	GLOB_ITEM_STR("ts2phc.nmea_remote_host", ""),
+	GLOB_ITEM_STR("ts2phc.nmea_remote_port", ""),
+	GLOB_ITEM_STR("ts2phc.nmea_serialport", "/dev/ttyS0"),
+	PORT_ITEM_INT("ts2phc.pin_index", 0, 0, INT_MAX),
+	GLOB_ITEM_INT("ts2phc.pulsewidth", 500000000, 1000000, 999000000),
 	PORT_ITEM_ENU("tsproc_mode", TSPROC_FILTER, tsproc_enu),
 	GLOB_ITEM_INT("twoStepFlag", 1, 0, 1),
 	GLOB_ITEM_INT("tx_timestamp_timeout", 1, 1, INT_MAX),
 	PORT_ITEM_INT("udp_ttl", 1, 1, 255),
 	PORT_ITEM_INT("udp6_scope", 0x0E, 0x00, 0x0F),
 	GLOB_ITEM_STR("uds_address", "/var/run/ptp4l"),
+	PORT_ITEM_INT("unicast_listen", 0, 0, 1),
+	PORT_ITEM_INT("unicast_master_table", 0, 0, INT_MAX),
+	PORT_ITEM_INT("unicast_req_duration", 3600, 10, INT_MAX),
 	GLOB_ITEM_INT("use_syslog", 1, 0, 1),
 	GLOB_ITEM_STR("userDescription", ""),
 	GLOB_ITEM_INT("utc_offset", CURRENT_UTC_OFFSET, 0, INT_MAX),
 	GLOB_ITEM_INT("verbose", 0, 0, 1),
+	GLOB_ITEM_INT("write_phase_mode", 0, 0, 1),
 };
 
+static struct unicast_master_table *current_uc_mtab;
+
 static enum parser_result
 parse_fault_interval(struct config *cfg, const char *section,
 		     const char *option, const char *value);
@@ -317,10 +405,103 @@ static void config_item_free(void *ptr)
 	free(ci);
 }
 
+static int config_switch_unicast_mtab(struct config *cfg, int idx, int line_num)
+{
+	struct unicast_master_table *table;
+
+	if (idx < 1) {
+		fprintf(stderr, "line %d: table_id %d is out of range. "
+			"Must be in the range %d to %d\n",
+			line_num, idx, 1, INT_MAX);
+		return -1;
+	}
+	STAILQ_FOREACH(table, &cfg->unicast_master_tables, list) {
+		if (table->table_index == idx) {
+			fprintf(stderr, "line %d: table_id %d already taken\n",
+				line_num, idx);
+			return -1;
+		}
+	}
+	table = calloc(1, sizeof(*table));
+	if (!table) {
+		fprintf(stderr, "low memory\n");
+		return -1;
+	}
+	STAILQ_INIT(&table->addrs);
+	table->table_index = idx;
+	memset(&table->peer_addr.portIdentity, 0xff,
+	       sizeof(table->peer_addr.portIdentity));
+	STAILQ_INSERT_TAIL(&cfg->unicast_master_tables, table, list);
+	current_uc_mtab = table;
+	return 0;
+}
+
+static int config_unicast_mtab_address(enum transport_type type, char *address,
+				       int line_num)
+{
+	struct unicast_master_address *item;
+
+	if (!current_uc_mtab) {
+		fprintf(stderr, "line %d: missing table_id\n", line_num);
+		return -1;
+	}
+	item = calloc(1, sizeof(*item));
+	if (!item) {
+		fprintf(stderr, "low memory\n");
+		return -1;
+	}
+	if (str2addr(type, address, &item->address)) {
+		fprintf(stderr, "line %d: bad address\n", line_num);
+		free(item);
+		return -1;
+	}
+	memset(&item->portIdentity, 0xff, sizeof(item->portIdentity));
+	item->type = type;
+	STAILQ_INSERT_TAIL(&current_uc_mtab->addrs, item, list);
+	current_uc_mtab->count++;
+
+	return 0;
+}
+
+static int config_unicast_mtab_peer(char *address, int line_num)
+{
+	if (!current_uc_mtab) {
+		fprintf(stderr, "line %d: missing table_id\n", line_num);
+		return -1;
+	}
+	if (current_uc_mtab->peer_name) {
+		free(current_uc_mtab->peer_name);
+	}
+	current_uc_mtab->peer_name = strdup(address);
+	if (!current_uc_mtab->peer_name) {
+		fprintf(stderr, "low memory\n");
+		return -1;
+	}
+	return 0;
+}
+
+static int config_unicast_mtab_query_interval(int lqi, int line_num)
+{
+	if (!current_uc_mtab) {
+		fprintf(stderr, "line %d: missing table_id\n", line_num);
+		return -1;
+	}
+	if (lqi < INT8_MIN || lqi > INT8_MAX) {
+		fprintf(stderr, "line %d: logQueryInterval %d out of range\n",
+			line_num, lqi);
+		return -1;
+	}
+	current_uc_mtab->logQueryInterval = lqi;
+	return 0;
+}
+
 static enum parser_result parse_section_line(char *s, enum config_section *section)
 {
 	if (!strcasecmp(s, "[global]")) {
 		*section = GLOBAL_SECTION;
+	} else if (!strcasecmp(s, "[unicast_master_table]")) {
+		*section = UC_MTAB_SECTION;
+		current_uc_mtab = NULL;
 	} else if (s[0] == '[') {
 		char c;
 		*section = PORT_SECTION;
@@ -461,6 +642,39 @@ static enum parser_result parse_fault_interval(struct config *cfg,
 	return NOT_PARSED;
 }
 
+static int parse_unicast_mtab_line(struct config *cfg, char *line, int line_num)
+{
+	char address[64 + 1] = {0}, transport[16 + 1] = {0};
+	enum transport_type type = TRANS_UDS;
+	struct config_enum *cte;
+	int cnt, lqi, table_id;
+
+	cnt = sscanf(line, " table_id %d", &table_id);
+	if (cnt == 1) {
+		return config_switch_unicast_mtab(cfg, table_id, line_num);
+	}
+	cnt = sscanf(line, " logQueryInterval %d", &lqi);
+	if (cnt == 1) {
+		return config_unicast_mtab_query_interval(lqi, line_num);
+	}
+	cnt = sscanf(line, " peer_address %64s", address);
+	if (cnt == 1) {
+		return config_unicast_mtab_peer(address, line_num);
+	}
+	cnt = sscanf(line, " %16s %64s", transport, address);
+	if (cnt != 2) {
+		fprintf(stderr, "bad master table at line %d\n", line_num);
+		return -1;
+	}
+	for (cte = nw_trans_enu; cte->label; cte++) {
+		if (!strcasecmp(cte->label, transport)) {
+			type = cte->value;
+			break;
+		}
+	}
+	return config_unicast_mtab_address(type, address, line_num);
+}
+
 static enum parser_result parse_setting_line(char *line,
 					     const char **option,
 					     const char **value)
@@ -502,7 +716,7 @@ static void check_deprecated_options(const char **option)
 	}
 }
 
-static struct option *config_alloc_longopts(struct config *cfg)
+static struct option *config_alloc_longopts(void)
 {
 	struct config_item *ci;
 	struct option *opts;
@@ -521,7 +735,7 @@ static struct option *config_alloc_longopts(struct config *cfg)
 	return opts;
 }
 
-int config_read(char *name, struct config *cfg)
+int config_read(const char *name, struct config *cfg)
 {
 	enum config_section current_section = UNKNOWN_SECTION;
 	enum parser_result parser_res;
@@ -571,6 +785,13 @@ int config_read(char *name, struct config *cfg)
 			continue;
 		}
 
+		if (current_section == UC_MTAB_SECTION) {
+			if (parse_unicast_mtab_line(cfg, line, line_num)) {
+				goto parse_error;
+			}
+			continue;
+		}
+
 		if (current_section == UNKNOWN_SECTION) {
 			fprintf(stderr, "line %d is not in a section\n", line_num);
 			goto parse_error;
@@ -579,15 +800,15 @@ int config_read(char *name, struct config *cfg)
 		if (parse_setting_line(line, &option, &value)) {
 			fprintf(stderr, "could not parse line %d in %s section\n",
 				line_num, current_section == GLOBAL_SECTION ?
-				"global" : current_port->name);
+				"global" : interface_name(current_port));
 			goto parse_error;
 		}
 
 		check_deprecated_options(&option);
 
 		parser_res = parse_item(cfg, 0, current_section == GLOBAL_SECTION ?
-					NULL : current_port->name, option, value);
-
+					NULL : interface_name(current_port),
+					option, value);
 		switch (parser_res) {
 		case PARSED_OK:
 			break;
@@ -595,7 +816,7 @@ int config_read(char *name, struct config *cfg)
 			fprintf(stderr, "unknown option %s at line %d in %s section\n",
 				option, line_num,
 				current_section == GLOBAL_SECTION ? "global" :
-				current_port->name);
+				interface_name(current_port));
 			goto parse_error;
 		case BAD_VALUE:
 			fprintf(stderr, "%s is a bad value for option %s at line %d\n",
@@ -621,23 +842,23 @@ parse_error:
 	return -2;
 }
 
-struct interface *config_create_interface(char *name, struct config *cfg)
+struct interface *config_create_interface(const char *name, struct config *cfg)
 {
 	struct interface *iface;
+	const char *ifname;
 
 	/* only create each interface once (by name) */
 	STAILQ_FOREACH(iface, &cfg->interfaces, list) {
-		if (0 == strncmp(name, iface->name, MAX_IFNAME_SIZE))
+		ifname = interface_name(iface);
+		if (0 == strncmp(name, ifname, MAX_IFNAME_SIZE))
 			return iface;
 	}
 
-	iface = calloc(1, sizeof(struct interface));
+	iface = interface_create(name);
 	if (!iface) {
 		fprintf(stderr, "cannot allocate memory for a port\n");
 		return NULL;
 	}
-
-	strncpy(iface->name, name, MAX_IFNAME_SIZE);
 	STAILQ_INSERT_TAIL(&cfg->interfaces, iface, list);
 	cfg->n_interfaces++;
 
@@ -656,8 +877,9 @@ struct config *config_create(void)
 		return NULL;
 	}
 	STAILQ_INIT(&cfg->interfaces);
+	STAILQ_INIT(&cfg->unicast_master_tables);
 
-	cfg->opts = config_alloc_longopts(cfg);
+	cfg->opts = config_alloc_longopts();
 	if (!cfg->opts) {
 		free(cfg);
 		return NULL;
@@ -701,11 +923,24 @@ fail:
 
 void config_destroy(struct config *cfg)
 {
+	struct unicast_master_address *address;
+	struct unicast_master_table *table;
 	struct interface *iface;
 
 	while ((iface = STAILQ_FIRST(&cfg->interfaces))) {
 		STAILQ_REMOVE_HEAD(&cfg->interfaces, list);
-		free(iface);
+		interface_destroy(iface);
+	}
+	while ((table = STAILQ_FIRST(&cfg->unicast_master_tables))) {
+		while ((address = STAILQ_FIRST(&table->addrs))) {
+			STAILQ_REMOVE_HEAD(&table->addrs, list);
+			free(address);
+		}
+		if (table->peer_name) {
+			free(table->peer_name);
+		}
+		STAILQ_REMOVE_HEAD(&cfg->unicast_master_tables, list);
+		free(table);
 	}
 	hash_destroy(cfg->htab, config_item_free);
 	free(cfg->opts);
