@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <sys/queue.h>
 #include <time.h>
+#include <asm/byteorder.h>
 
 #include "address.h"
 #include "ddt.h"
@@ -55,6 +56,13 @@
 #define PTP_TIMESCALE  (1<<3)
 #define TIME_TRACEABLE (1<<4)
 #define FREQ_TRACEABLE (1<<5)
+#define SYNC_UNCERTAIN (1<<6)
+
+/*
+ * Signaling interval special values. For more info look at 802.1AS table 10-11
+ */
+#define SIGNAL_NO_CHANGE   -128
+#define SIGNAL_SET_INITIAL 126
 
 enum timestamp_type {
 	TS_SOFTWARE,
@@ -227,10 +235,6 @@ struct ptp_message {
 	 * pointers to the appended TLVs.
 	 */
 	TAILQ_HEAD(tlv_list, tlv_extra) tlv_list;
-	/**
-	 * Contains the number of TLVs in the suffix.
-	 */
-	int tlv_count;
 };
 
 /**
@@ -279,6 +283,13 @@ struct tlv_extra *msg_tlv_append(struct ptp_message *msg, int length);
  */
 void msg_tlv_attach(struct ptp_message *msg, struct tlv_extra *extra);
 
+/*
+ * Return the number of TLVs attached to a message.
+ * @param msg  A message obtained using @ref msg_allocate().
+ * @return     The number of attached TLVs.
+ */
+int msg_tlv_count(struct ptp_message *msg);
+
 /**
  * Obtain the transportSpecific field from a message.
  * @param m  Message to test.
@@ -295,7 +306,7 @@ static inline UInteger8 msg_transport_specific(struct ptp_message *m)
  * @param m  Message to test.
  * @return   The value of the messageType field.
  */
-static inline int msg_type(struct ptp_message *m)
+static inline int msg_type(const struct ptp_message *m)
 {
 	return m->header.tsmt & 0x0f;
 }
@@ -316,6 +327,26 @@ struct ptp_message *msg_allocate(void);
  */
 void msg_cleanup(void);
 
+/**
+ * Duplicate a message instance.
+ *
+ * This function accepts a message in network byte order and returns a
+ * duplicate in host byte.
+ *
+ * Messages are reference counted, and newly allocated messages have a
+ * reference count of one. Allocated messages are freed using the
+ * function @ref msg_put().
+ *
+ * @param msg  A message obtained using @ref msg_allocate().
+ *             The passed message must be in network byte order, not
+ *             having been passed to @ref msg_post_recv().
+ *
+ * @return     Pointer to a message on success, NULL otherwise.
+ *             The returned message will be in host byte order, having
+ *             been passed to @ref msg_post_recv().
+ */
+struct ptp_message *msg_duplicate(struct ptp_message *msg, int cnt);
+
 /**
  * Obtain a reference to a message, increasing its reference count by one.
  * @param m A message obtained using @ref msg_allocate().
@@ -374,6 +405,16 @@ static inline int msg_sots_valid(struct ptp_message *m)
 	return !tmv_is_zero(m->hwts.ts);
 }
 
+/**
+ * Test whether a message is a unicast message.
+ * @param m  Message to test.
+ * @return   One if the message is unicast, zero otherwise.
+ */
+static inline Boolean msg_unicast(struct ptp_message *m)
+{
+	return field_is_set(m, 0, UNICAST);
+}
+
 /**
  * Work around buggy 802.1AS switches.
  */
@@ -394,11 +435,17 @@ static inline Boolean one_step(struct ptp_message *m)
 /**
  * Convert a 64 bit word into network byte order.
  */
-int64_t host2net64(int64_t val);
+static inline int64_t host2net64(int64_t val)
+{
+	return __cpu_to_be64(val);
+}
 
 /**
  * Convert a 64 bit word into host byte order.
  */
-int64_t net2host64(int64_t val);
+static inline int64_t net2host64(int64_t val)
+{
+	return __be64_to_cpu(val);
+}
 
 #endif
