@@ -150,7 +150,7 @@ static int raw_close(struct transport *t, struct fdarray *fda)
 }
 
 static int open_socket(const char *name, int event, unsigned char *ptp_dst_mac,
-		       unsigned char *p2p_dst_mac)
+		       unsigned char *p2p_dst_mac, int socket_priority)
 {
 	struct sockaddr_ll addr;
 	int fd, index;
@@ -176,6 +176,13 @@ static int open_socket(const char *name, int event, unsigned char *ptp_dst_mac,
 		pr_err("setsockopt SO_BINDTODEVICE failed: %m");
 		goto no_option;
 	}
+
+	if (socket_priority > 0 &&
+	    setsockopt(fd, SOL_SOCKET, SO_PRIORITY, &socket_priority,
+		       sizeof(socket_priority))) {
+		pr_err("setsockopt SO_PRIORITY failed: %m");
+		goto no_option;
+	}
 	if (raw_configure(fd, event, index, ptp_dst_mac, p2p_dst_mac, 1))
 		goto no_option;
 
@@ -205,10 +212,11 @@ static int raw_open(struct transport *t, struct interface *iface,
 	struct raw *raw = container_of(t, struct raw, t);
 	unsigned char ptp_dst_mac[MAC_LEN];
 	unsigned char p2p_dst_mac[MAC_LEN];
-	int efd, gfd;
-	char *str, *name;
+	int efd, gfd, socket_priority;
+	const char *name;
+	char *str;
 
-	name = iface->ts_label;
+	name = interface_label(iface);
 	str = config_get_string(t->cfg, name, "ptp_dst_mac");
 	if (str2mac(str, ptp_dst_mac)) {
 		pr_err("invalid ptp_dst_mac %s", str);
@@ -225,11 +233,13 @@ static int raw_open(struct transport *t, struct interface *iface,
 	if (sk_interface_macaddr(name, &raw->src_addr))
 		goto no_mac;
 
-	efd = open_socket(name, 1, ptp_dst_mac, p2p_dst_mac);
+	socket_priority = config_get_int(t->cfg, "global", "socket_priority");
+
+	efd = open_socket(name, 1, ptp_dst_mac, p2p_dst_mac, socket_priority);
 	if (efd < 0)
 		goto no_event;
 
-	gfd = open_socket(name, 0, ptp_dst_mac, p2p_dst_mac);
+	gfd = open_socket(name, 0, ptp_dst_mac, p2p_dst_mac, socket_priority);
 	if (gfd < 0)
 		goto no_general;
 
@@ -269,7 +279,7 @@ static int raw_recv(struct transport *t, int fd, void *buf, int buflen,
 	buflen += hlen;
 	hdr = (struct eth_hdr *) ptr;
 
-	cnt = sk_receive(fd, ptr, buflen, addr, hwts, 0);
+	cnt = sk_receive(fd, ptr, buflen, addr, hwts, MSG_DONTWAIT);
 
 	if (cnt >= 0)
 		cnt -= hlen;
@@ -290,15 +300,27 @@ static int raw_recv(struct transport *t, int fd, void *buf, int buflen,
 	return cnt;
 }
 
-static int raw_send(struct transport *t, struct fdarray *fda, int event,
-		    int peer, void *buf, int len, struct address *addr,
-		    struct hw_timestamp *hwts)
+static int raw_send(struct transport *t, struct fdarray *fda,
+		    enum transport_event event, int peer, void *buf, int len,
+		    struct address *addr, struct hw_timestamp *hwts)
 {
 	struct raw *raw = container_of(t, struct raw, t);
 	ssize_t cnt;
-	int fd = event ? fda->fd[FD_EVENT] : fda->fd[FD_GENERAL];
 	unsigned char pkt[1600], *ptr = buf;
 	struct eth_hdr *hdr;
+	int fd = -1;
+
+	switch (event) {
+	case TRANS_GENERAL:
+		fd = fda->fd[FD_GENERAL];
+		break;
+	case TRANS_EVENT:
+	case TRANS_ONESTEP:
+	case TRANS_P2P1STEP:
+	case TRANS_DEFER_EVENT:
+		fd = fda->fd[FD_EVENT];
+		break;
+	}
 
 	ptr -= sizeof(*hdr);
 	len += sizeof(*hdr);
@@ -314,8 +336,7 @@ static int raw_send(struct transport *t, struct fdarray *fda, int event,
 
 	cnt = send(fd, ptr, len, 0);
 	if (cnt < 1) {
-		pr_err("send failed: %d %m", errno);
-		return cnt;
+		return -errno;
 	}
 	/*
 	 * Get the time stamp right away.
