@@ -52,13 +52,14 @@ static int uds_close(struct transport *t, struct fdarray *fda)
 	return 0;
 }
 
-static int uds_open(struct transport *t, const char *name, struct fdarray *fda,
+static int uds_open(struct transport *t, struct interface *iface, struct fdarray *fda,
 		    enum timestamp_type tt)
 {
-	int fd, err;
-	struct sockaddr_un sa;
-	struct uds *uds = container_of(t, struct uds, t);
 	char *uds_path = config_get_string(t->cfg, NULL, "uds_address");
+	struct uds *uds = container_of(t, struct uds, t);
+	const char *name = interface_name(iface);
+	struct sockaddr_un sa;
+	int fd, err;
 
 	fd = socket(AF_LOCAL, SOCK_DGRAM, 0);
 	if (fd < 0) {
@@ -107,9 +108,9 @@ static int uds_recv(struct transport *t, int fd, void *buf, int buflen,
 	return cnt;
 }
 
-static int uds_send(struct transport *t, struct fdarray *fda, int event,
-		    int peer, void *buf, int buflen, struct address *addr,
-		    struct hw_timestamp *hwts)
+static int uds_send(struct transport *t, struct fdarray *fda,
+		    enum transport_event event, int peer, void *buf, int buflen,
+		    struct address *addr, struct hw_timestamp *hwts)
 {
 	int cnt, fd = fda->fd[FD_GENERAL];
 	struct uds *uds = container_of(t, struct uds, t);
@@ -118,8 +119,8 @@ static int uds_send(struct transport *t, struct fdarray *fda, int event,
 		addr = &uds->address;
 
 	cnt = sendto(fd, buf, buflen, 0, &addr->sa, addr->len);
-	if (cnt <= 0 && errno != ECONNREFUSED) {
-		pr_err("uds: sendto failed: %m");
+	if (cnt < 1) {
+		return -errno;
 	}
 	return cnt;
 }
@@ -143,4 +144,3 @@ struct transport *uds_transport_create(void)
 	uds->t.release = uds_release;
 	return &uds->t;
 }
-
