@@ -23,8 +23,10 @@
 #include <string.h>
 #include <time.h>
 
+#include "address.h"
 #include "ddt.h"
 #include "ether.h"
+#include "transport.h"
 
 #define MAX_PRINT_BYTES 16
 #define BIN_BUF_SIZE (MAX_PRINT_BYTES * 3 + 1)
@@ -39,6 +41,22 @@ extern const char *ps_str[];
  */
 extern const char *ev_str[];
 
+/**
+ * Gets a human-readable string for a given timestamp type.
+ * @param ts    Timestamp type.
+ * @return      Human-readable rendering if TS is valid, otherwise "???".
+ */
+const char *ts_str(enum timestamp_type ts);
+
+/**
+ * Compares two binary addresses for equality.
+ * @param type  One of the enumerated transport types.
+ * @param a     One address to compare.
+ * @param b     The second address to compare.
+ * @return      One if the addresses are identical, zero otherwise.
+ */
+int addreq(enum transport_type type, struct address *a, struct address *b);
+
 static inline uint16_t align16(uint16_t *p)
 {
 	uint16_t v;
@@ -59,6 +77,18 @@ char *bin2str_impl(Octet *data, int len, char *buf, int buf_len);
  */
 char *cid2str(struct ClockIdentity *id);
 
+/**
+ * Compare two clock identities for equality.
+ *
+ * @param a  First clock identity.
+ * @param b  Second clock identity.
+ * @return   1 if identities are equal, 0 otherwise.
+ */
+static inline int cid_eq(struct ClockIdentity *a, struct ClockIdentity *b)
+{
+	return memcmp(a, b, sizeof(*a)) == 0;
+}
+
 /**
  * Counts the number of occurrences of a given character.
  * @param str  String to evaluate.
@@ -80,6 +110,42 @@ char *pid2str(struct PortIdentity *id);
 
 char *portaddr2str(struct PortAddress *addr);
 
+/**
+ * Closes a dynamic posix clock.
+ * @param clock  A clock ID obtained via posix_clock_close().
+ */
+void posix_clock_close(clockid_t clock);
+
+/**
+ * Opens a dynamic posix clock by name.
+ * @param device     The PHC character device or network interface to open.
+ * @param phc_index  Returns the PHC index, if any.
+ * @return           A valid clock ID on success or CLOCK_INVALID otherwise.
+ */
+clockid_t posix_clock_open(const char *device, int *phc_index);
+
+/**
+ * Compare two port identities for equality.
+ *
+ * @param a  First port identity.
+ * @param b  Second port identity.
+ * @return   1 if identities are equal, 0 otherwise.
+ */
+static inline int pid_eq(const struct PortIdentity *a,
+			 const struct PortIdentity *b)
+{
+	return memcmp(a, b, sizeof(*a)) == 0;
+}
+
+/**
+ * Convert a string containing a network address into binary form.
+ * @param type  The network transport type of the address.
+ * @param s     String in human readable form.
+ * @param addr  Pointer to a buffer to hold the result.
+ * @return Zero on success, or -1 if the string is incorrectly formatted.
+ */
+int str2addr(enum transport_type type, const char *s, struct address *addr);
+
 /**
  * Scan a string containing a MAC address and convert it into binary form.
  *
@@ -89,6 +155,15 @@ char *portaddr2str(struct PortAddress *addr);
  */
 int str2mac(const char *s, unsigned char mac[MAC_LEN]);
 
+/**
+ * Scan a string containing a clock identity and convert it into binary form.
+ *
+ * @param s       String in human readable form.
+ * @param result  Pointer to a buffer to hold the result.
+ * @return Zero on success, or -1 if the string is incorrectly formatted.
+ */
+int str2cid(const char *s, struct ClockIdentity *result);
+
 /**
  * Scan a string containing a port identity and convert it into binary form.
  *
